<?php
// Copyright (c) 2022 NetApp, Inc.
// All rights reserved.
//
// The PIDs in the 'lun' table were incorrectly initialized, causing APS jobs
// storing lun rows in RDB to hit problems between 9.5 and 9.6. Prior to
// upgrade to 9.7, where the now corrupt rows need to be converted from legacy
// RDB to SQL RDB, ensure there are no outstanding jobs and no stale records in
// RDB.
//
// I think we should also just leave this ANDU task around forever. The APS
// tables are designed to store random table rows, so while unlikely with all
// the new build checkers, it's possible we're going to be exposed to this type
// of issue in the future too. It shouldn't be a problem to require
// provisioning jobs to complete before an upgrade. It should be at most a 10
// minute delay, and will likely prevent the jobs from running into problems
// around nodes being offline anyway.

$result = dotsql_query('SELECT id FROM jm_jobs_table WHERE category=APS');
if (dotsql_num_rows($result) > 0) {
    echo "<Alert>Application provisioning jobs are still in progress. They must be allowed to complete before attempting NDU.</Alert>";
    echo "<Action>Use the \"job show -category APS\" command to check for the state of running jobs.</Action>";
    echo "<Advice>Wait for all category 'APS' jobs to complete and be purged from the system.</Advice>";
    echo "<RC>error</RC>";
    exit;
}

# In SQL proper, a WHERE clause is not required on a DELETE. In dotsql, it is
# required, so we use a WHERE that should match all records.
# https://opengrok-prd.eng.netapp.com/source/xref/dev/dotws/src/query/smdb/smdb_statement.cc#233
function clearTable($tableName, $cond) {
    $result = dotsql_query("DELETE FROM $tableName WHERE $cond");
    if ($result == false) {
        echo "<Alert>An application provisioning job failed to remove temporary data from the system. This data must be manually deleted before attempting NDU.</Alert>";
        echo "<Action>Contact technical support.</Action>";
        echo "<RC>error</RC>";
        exit;
    }
}

$min_uuid = '00000000-0000-0000-0000-000000000000';

# dotsql does not require field names with - to be quoted like sqlite.
clearTable('aps_smf_actions', "action-id>=$min_uuid");
clearTable('aps_smf_tasks', 'job-id>=0');
clearTable('aps_smf_rollback', "todo-id>=$min_uuid");
clearTable('aps_provision_lun', 'job-id>=0');
clearTable('aps_provision_volume', 'job-id>=0');
clearTable('aps_provision_qos_policy', 'job-id>=0');

exit;
?>

