<?php
// Copyright (c) 2023 NetApp, Inc.
// All rights reserved.
//
// In a special case, if a cluster ran certain X-Builds, the pubsub config  may
// have been initialized with Non-Production values.  This script will purge
// the non-production data from the config tables, allowing them to be re-created
// from correct production defaults.
//
// I think we should also just leave this ANDU task around forever. The sript
// will safeguard the purge by checking that the manditory port number is set.

# In SQL proper, a WHERE clause is not required on a DELETE. In dotsql, it is
# required, so we use a WHERE that should match all records.
# https://opengrok-prd.eng.netapp.com/source/xref/dev/dotws/src/query/smdb/smdb_statement.cc#233
function clearTable($tableName, $cond) {
    $result = dotsql_query("DELETE FROM $tableName WHERE $cond");
    if ($result == false) {
        echo "<Alert>An application provisioning job failed to remove invalid AMQP configuration data from the ",$tableName," table. This data must be manually deleted before attempting NDU.</Alert>";
        echo "<Action>Contact technical support.</Action>";
        echo "<RC>error</RC>";
        exit;
    }
}

$query = "SELECT port FROM pubsub_config_broker";
$result = dotsql_query($query);
if ($row = dotsql_fetch_object($result)) {
    # The kernel sender is hard-coded to port 5672. A non-production release included a configuration for port 948.
    # This prevents the brokers from going to the 'ready' state, and causes constant reconnection pressure in the
    # kernel.  To repair this, it is best to delete the broker + router configurations and let them repopulate with
    # the values from the default_imp for each table.
    if ($row->port == "948") {
        # dotsql does not require field names with - to be quoted like sqlite.
        clearTable('pubsub_config_broker', "port=948");
        clearTable('pubsub_config_router', 'connector-port=948');

        # Reset the AMQP brokers to refresh the active broker lists in the config
        dotsql_query("CALL pubsub_ha_control_restart_brokers( node=local )");

        # This value indicates that a non-production set of defaults has been discovered.
        # These values do not work, and must be erased to allow standard defaults to get injected.
        echo "<Alert>Invalid AMQP configuration was detected and fixed.</Alert>";
        echo "<Action>No action required.</Action>";
        echo "<Advice>No action required.</Advice>";
        echo "<RC>warning</RC>";
        exit;
    }
}

exit;
?>

