<?php
//
//  Copyright (c) 2014 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

//
// 1.Check if aggregates are offline.
// 2.Check for aggregate with ignore-inconsistent field turned on.
// 3.Check for aggregates with uninitialized parity.


include 'mcc_util.php';
global $mccconfigured;
define("mccconfigured","false");
global $mccswitchedover;
$offlineaggrfound = false;
$uninitparity = false;
$inconsistent = false;

$offlineaggr_alert = "";
$inconsistent_alert = "";
$uninitparity_alert = "";

$offlineaggr_action = "";
$inconsistent_action = "";
$uninitparity_action = "";

$offlineaggr_advice = "";
$inconsistent_advice = "";
$uninitparity_advice = "";

$options = getoptreq('',array("major:", "downgrade::"));
$majorupgrade = $options["major"];

if(!isValidMCCState()) {
    exit;
}

$queryaggr = "SELECT cluster-id FROM aggr WHERE state != online";
$resultaggr = dotsql_query($queryaggr);

if ($mccconfigured) {
    while ($rowaggr = dotsql_fetch_object($resultaggr)) {
        $clusterid = $rowaggr->{'cluster-id'};
        $querycid = "SELECT uuid from cluster_id";
        $resultcid = dotsql_query($querycid);
        if($rowcid = dotsql_fetch_object($resultcid)) {
            if($rowcid->uuid != $clusterid) {
                if(!$mccswitchedover){
                    // No aggrs are shown offline when mcc is in normal state. So should not enter here
                    continue;
                }
                else {
                    // DR site root aggregates will be shown as offline during switched-over, heal-aggregates, heal-root-aggregates phases. Skip them.
                    // Only reliable attribute to check if it is DR site root aggr is ha-policy. The value could be null after heal-root-aggregates.
                    $hapolicy= $rowaggr->{'ha-policy'};
                    if($hapolicy == 'cfo' || $hapolicy == '') {
                        continue;
                    }
                    else {
                        $offlineaggrfound = true;
                        break;
                    }
                }
            }
            else {
                //Found a local site offline aggregate
                $offlineaggrfound = true;
                break;
            }
        }
    }
} elseif (dotsql_num_rows($resultaggr)) {
    //echo "Not an MCC configuration and there are some offline aggregates...";
    $offlineaggrfound = true;
}

if ($offlineaggrfound) {
    $offlineaggr_alert = "Not all aggregates are online. Ensure all aggregates are online before attempting NDU. ";
    $offlineaggr_action = "To check for offline aggregates run the \"storage aggregate show -state !online\" command. ";
    $offlineaggr_advice = "Check for offline aggregates in the cluster. ";
}

//Ignore inconsistent check should be off
$queryaggr = "SELECT aggregate FROM aggr WHERE ignore-inconsistent = 'on'";
$resultaggr = dotsql_query($queryaggr);

while ($rowaggr = dotsql_fetch_object($resultaggr)) {
    $inconsistent_alert = "One or more aggregates have been marked to ignore inconsistent checks. Ensure all aggregates have turned on inconsistent checks before attempting NDU. ";
    $inconsistent_action = "To check for aggregates with ignore inconsistent checks turned on, run the \"storage aggregate show -ignore-inconsistent on\" command. ";
    $inconsistent_advice = "Check for aggregates in the cluster which are marked to ignore inconsistent checks. ";
    $inconsistent = true;
	break;
}

//Check for parity uninit'd
$queryaggr = "SELECT raidstatus FROM aggr";
$resultaggr = dotsql_query($queryaggr);
$key = "parity uninit";

while ($rowaggr = dotsql_fetch_object($resultaggr)) {
   $status = $rowaggr->{'raidstatus'};
   if (strpos($status, $key) == true) {
        $uninitparity_alert = "Some aggregates have parity as uninitialized. ";
        $uninitparity_action = "To check for aggregates with uninitialized parity, run the \"storage aggregate show\" command. Contact technical support for assistance. ";
        $uninitparity_advice = "Contact technical support for assistance. ";
        $uninitparity = true;
		break;
   }
}

if ($inconsistent || $offlineaggrfound || $uninitparity) {
	$alert = "<Alert>".$offlineaggr_alert.$inconsistent_alert.$uninitparity_alert."</Alert>";
	$action = "<Action>".$offlineaggr_action.$inconsistent_action.$uninitparity_action."</Action>";
	$advice = "<Advice>".$offlineaggr_advice.$inconsistent_advice.$uninitparity_advice."</Advice>";
	echo $alert;
	echo $action;
	echo $advice;

	//If there are offline aggrs, irrespective of the other 2 checks, set the status as error or warning depending on whether its major upgrade or not.
	//If there are no offline aggrs, then if the other 2 checks are true, then its always warning.
	if ($offlineaggrfound && $majorupgrade == "true") {
        echo "<RC>error</RC>";
	} else {
        echo "<RC>warning</RC>";
	}
}

exit;
?>
