<?php
//
//  Copyright (c) 2017-2018 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

//
// Block upgrade if (1) external keymanager is configured, (2) the release from
// which we are upgrading is FT or KB, and (3) this is Cloud ONTAP since they
// use legacy KMIP client.

$FT = "9.3";
$KB = "9.4";

/**
 * Returns true if the bootarg is set to true, false otherwise.
 */
function isBootargTrue($val)
{
    return ((strcasecmp($val, "true") == 0) || ($val == "1"));
}

/**
 * Returns true if the given bootargVal matches the given expectedVal
 * (case insensitive), false otherwise.
 */
function bootargMatches($bootargVal, $expectedVal)
{
    return (strcasecmp($bootargVal, $expectedVal) == 0);
}

/**
 * Reads the kenv with the given name and returns its value.  If a kenv with
 * the given kenvNane does not exist, returns the empty string.
 */
function readKenv($kenvName)
{
    $value = "";

    $result = dotsql_query("SELECT value FROM debug_kenv WHERE variable = $kenvName");
    if ($row = dotsql_fetch_object($result)) {
        $value = $row->value;
    }

    return $value;
}

/**
 * Returns true if we're running on Cloud ONTAP, false otherwise.
 */
function isCloudOntap()
{
    $bootargAws = readKenv("bootarg.vm.aws");
    $bootargVmProvider = readKenv("bootarg.vm.provider");

    return (isBootargTrue($bootargAws) ||
            bootargMatches($bootargVmProvider, "azure"));
}

if (isCloudOntap()) {
    $query = "SELECT key-store FROM cluster_kdb WHERE key-store = external";
    $result = dotsql_query($query);

    if (dotsql_num_rows($result) > 0) {
        $query = "SELECT version FROM software_image WHERE iscurrent = true AND (version LIKE (%$KB% | %$FT%))";
        $result = dotsql_query($query);

        if (dotsql_num_rows($result) > 0) {
            echo "<Alert>";
            echo "External key management is configured on the cluster.";
            echo "</Alert><Action>";
            echo "Please unconfigure external key management before updating ";
            echo "any nodes. To unconfigure, run the (privilege: advanced) ";
            echo "'security key-manager delete-kmip-config' command.";
            echo "</Action><Advice>";
            echo "Please unconfigure external key management before updating ";
            echo "any nodes.";
            echo "</Advice>";
            echo "<RC>error</RC>";
        }
    }
}

exit;
?>
