<?php
//
// Copyright (c) 2023-2023 NetApp, Inc.
// All rights reserved.
//
// VERSION="1.0"
//
// This validation script checks for system DIMM status.  If there is any
// DIMM in degraded state (due to CECC count above threshold), a warning
// will be issued to request user to ensure they want to proceed to do
// software update.  There may be a small chance of DIMM failure during
// reboot.
//


//
// Messages issued to user/customer.
//
$action_msg = "Use the \"system health alert show\" command to check "
            . "system health.  Then, determine whether software upgrade "
            . "should continue.";
$advice_msg = "Ensure system hardware is healthy for continuation of "
            . "software upgrade.";
$error_msg = "Some DIMMs are in degraded state.  DIMM replacement might be needed.";

$failure = false;

function logDebugMessage($message, $log_error) {
    $dbgquery = "CALL add_scripting_log_message('validation_rules_health_monitor: $message',
                 $log_error)";
    $result = dotsql_query($dbgquery);
}

$query = "SELECT pds-id, name FROM nphm_dimm_mgwd_info WHERE status = degraded";
$result = dotsql_query($query);
if (dotsql_num_rows($result)) {
    $failure = true;
}

if ($failure) {
    logDebugMessage("HM DIMM failure detected.", 'false');
    echo "<Alert>$error_msg</Alert>";
    echo "<Action>$action_msg</Action>";
    echo "<Advice>$advice_msg</Advice>";
    echo "<RC>warning</RC>";
} else {
    logDebugMessage("HM DIMMs are ok", 'false');
}

exit;
?>
