<?php

// Copyright (c) 2020-2023 NetApp, Inc.
// All rights reserved.

// Check for missing keys for OKM/KMIP enabled systems.

$enc_key_alert = "One or more encryption keys are unavailable.";
$enc_key_action = "Restore missing encryption keys before starting ANDU. To check missing keys, run \"security key-manager key query -restored false\". To restore onboard key manager keys, run the \"security key-manager onboard sync\" command. To restore external key manager keys, run the \"security key-manager external restore\" command. To restore Azure Key Vault keys, run the \"security key-manager external azure restore\" command. To restore Google Cloud Key Management Service keys, run the \"security key-manager external gcp restore\" command. To restore Amazon Web Services Key Management Service keys, run the \"security key-manager external aws restore\" command.";
$enc_key_advice = "Restore missing encryption keys before starting ANDU.";



class aggrList {
	var $aggr_uuid;
	var $partner_name;

function __construct($aggr_uuid,$partner_name) {

	$this->aggr_uuid = $aggr_uuid;
	$this->partner_name = $partner_name;
}

function get_aggr_uuid() {
	return $this->aggr_uuid;
}

function get_partner_name() {
	return $this->partner_name;
}

function __destruct() {}
}

// For each node we will fetch its partner-name and aggrs and for each aggr we will call cryptomod api if :
// 1. It is a NAE aggr.
// 2. The aggr has encrypted volumes.
// For Non HA configs we will pass local-node to cryptomod api.

$aggr_list = array();
$node_query = dotsql_query("SELECT node from nodes");

while ($node_row = dotsql_fetch_object($node_query)) {
	$local_node = $node_row->{'node'};
	$partner_query = dotsql_query("SELECT 'partner-name' from haStateByID where node = $local_node");
	$partner_node = dotsql_fetch_object($partner_query)->{'partner-name'};
	if ($partner_node == "") {
		$partner_node = $local_node;
	}
	$aggrs_query = dotsql_query("SELECT uuid,aggregate,owner-name,encrypt-with-aggr-key from aggr where node = $local_node");
	while($aggr_row = dotsql_fetch_object($aggrs_query)) {
		$uuid = $aggr_row->{'uuid'};
		$aggr_name = $aggr_row->{'aggregate'};
		if ($aggr_row->{'encrypt-with-aggr-key'}) {
			array_push($aggr_list, new aggrList($uuid,$partner_node));
			continue;
		}
		$vol_query = "SELECT aggregate FROM volume WHERE 'encryption-state' != 'none' AND 'volume-style-extended' != 'flexgroup' AND aggregate='".$aggr_name."' LIMIT 1";
		$vol_query_result = dotsql_query($vol_query);
		if(dotsql_num_rows($vol_query_result) == 1) {
			array_push($aggr_list, new aggrList($uuid,$partner_node));
			continue;
		}
	}
}

//keymanager_vdek_key_on_dest table entries gets updated when
//key_available_on_dest api gets called again with same aggr uuid and partner name.
//When we rerun validation, keymanager_vdek_key_on_dest holds the previous entries of failures.
//So, our script gets failed and break.To fix this, script needs to call key_avaiable_on_dest api
//for all entries in aggr_list[] and then check result once the result is fully updated with new status.
//This is a fix for burt 1310488.

for ($count = 0; $count < sizeof($aggr_list); $count++) {
	$key_query_statement = "CALL keymanager_vdek_table_key_available_on_dest('".$aggr_list[$count]->get_aggr_uuid()."','".$aggr_list[$count]->get_partner_name()."')";
	$key_query = dotsql_query($key_query_statement);
}

$on_dest_result = dotsql_query("SELECT result FROM keymanager_vdek_key_on_dest WHERE result = 'false'");

if(dotsql_num_rows($on_dest_result) > 0){
    echo "<Alert>$enc_key_alert</Alert>";
    echo "<Action>$enc_key_action</Action>";
    echo "<Advice>$enc_key_advice </Advice>";
    echo "<RC>error</RC>";
}

exit;
?>
