<?php
//  Copyright (c) 2017 NetApp, Inc.
//  All rights reserved.

// NDU Compatibility for LDAP Fastbind users.
// VERSION="1.0"
// This script checks whether LDP Fastbind users are using unsecure connection to LDAP
// server and warns the user that, LDAP fastbind users using unsecure connection is not
// allowed post upgrade to YD. These fastbind users needs to be explicitly removed from
// the vserver using security login command
//
include 'mcc_util.php';

// All the validations only apply for major upgrades.
// Exit the validation in minor upgrades.
$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];
if($majorupgrade == "false") {
    exit;
}

// If the LDAP Fastbind user is using unsecure connection, then warn the user.
$unsecure = false;
$fastbindQuery = "SELECT vserverID FROM userprofile_all WHERE is-ldap-fastbind=yes";
$fastbindResult = dotsql_query($fastbindQuery);
while( $fastbindRow = dotsql_fetch_object($fastbindResult)) {
    $vserver = $fastbindRow->{'vserverID'};
    $vserverQuery = "SELECT vserver FROM vserver_by_name_data_admin WHERE id-unsigned=$vserver";
    $vserverResult = dotsql_query($vserverQuery);
    if($vserverRow = dotsql_fetch_object($vserverResult)){
	$vserverName = $vserverRow->{'vserver'};
	$ldapQuery = "SELECT client-config FROM ldap_config WHERE vserver=$vserverName";
	$ldapResult = dotsql_query($ldapQuery);
	if($ldapRow = dotsql_fetch_object($ldapResult)){
	    $clientConfig = $ldapRow->{'client-config'};
	    $ldapClientQuery = "SELECT use-start-tls, ldaps-enabled FROM ldap_client_config_ui WHERE vserver=$vserverName AND client-config=$clientConfig";
	    $ldapClientResult = dotsql_query($ldapClientQuery);
	    if($ldapClientRow = dotsql_fetch_object($ldapClientResult)){
		if(!($ldapClientRow->{'use-start-tls'} == "true" || $ldapClientRow->{'ldaps-enabled'} == "true")){
		    $unsecure = true;
		    break;
		}
	    }
	} else {
	    $unsecure = true;
	    break;
	}
    }
}

if ($unsecure) {
    $log_message = "LDAP Fastbind user using unsecure connection found.";
    logDebugMessage($log_message);
    warnUser($ldap_fastbind_user_with_unsecure_connection);
} else {
    $log_message = "LDAP Fastbind user using unsecure connection not found.";
    logDebugMessage($log_message);
}

//--------------------------------FUNCTION DEFINATION---------------------------
function logDebugMessage($log_message)
{
    $dbgquery = "CALL add_scripting_log_message('validation_rules_ldap_fastbind_users: $log_message','false')";
    $result = dotsql_query($dbgquery);
}

function warnUser($ldap_fastbind_user_with_unsecure_connection)
{
    logDebugMessage($log_message);
    echo "<Alert>";
    echo "It is recommended to use secure LDAP connection ";
    echo "while using ldap fastbind, as, the usename and ";
    echo "password is transmitted to LDAP servers. ";
    echo "</Alert><Action>";
    echo "Before retrying the upgrade, remove the ldap fastbind";
    echo "users using unsecure ldap connection with command ";
    echo "\"security login delete\" ";
    echo "</Action><Advice>";
    echo "Ensure that all ldap fastbind users using unsecure connection ";
    echo "are deleted before attempting to upgrade.";
    echo "</Advice>";
    echo "<RC>error</RC>";
}

//---------------------------------------------------------------------------------
exit;
?>
