<?php
//
//  Copyright (c) 2014 NetApp, Inc.
//  All rights reserved.
//
// VERSION="1.0"
// NDU Compatibility for SCON

// The first check is to verify a an HA pair has the same
// broadcast domains present on each node

include 'mcc_util.php';
include 'vserver_state_cache.php';

$MAX_BDOMAINS_TO_REPORT=3;
$VERBOSE=0;

function DisplayExecutionTime( $start, $finish, $msg )
{
    $delta = $finish - $start;
    if($VERBOSE==1) {
	echo "{$msg} took {$delta} seconds to run\n";
    }
}

// create a mapping of node's to their partner we use this later to determine
// which node to compare broadcast domains to
$nodeQuery = "SELECT node, partner-name, state FROM storage_failover";
$nodeResult = dotsql_query($nodeQuery);
while( $nodeRow = dotsql_fetch_object($nodeResult)) {
    // if state is empty, then HA is not turned on for the node
    // so it has no partner
    if( $nodeRow->{'state'} ) {
	$node = $nodeRow->{'node'};
	$partner[$node] = $nodeRow->{'partner-name'};
    }
}

$rows = 0;

// get a list of broadcast domains for each node
// this code handles the unlikely event that some nodes
// in the cluster have HA turned on and some have it
// turned off
$nodeQuery = "SELECT node_id_col, broadcast_domain_id_col, ipspace_id_col FROM port_table";
$startTime = microtime(true);
$nodeResult = dotsql_query($nodeQuery);
$queryTime = microtime(true);
DisplayExecutionTime( $startTime, $queryTime, "port table query");
while( $nodeRow = dotsql_fetch_object($nodeResult)) {
    $node = $nodeRow->{'node_id_col'};
    $rows++;
    // only get broadcast domains if the node has a partner
    if( $partner[$node] ){
	$ipspace = $nodeRow->{'ipspace_id_col'};
	$broadcastDomain = $nodeRow->{'broadcast_domain_id_col'};
	// handle the case where a port does not have an
	// associated broadcast domain. A port that is not
	// in a broadcast domain cannot host a lif so its
	// safe to treat the broadcast domain as though it
	// doesn't exist
	if( $broadcastDomain ){
	    // since we need the ipspace the broadcast domain
	    // resides in for reporting, we stash it away
	    // as an indicator a broadcast domain exists
	    // for that node
	    $nodeBD[$node][$ipspace][$broadcastDomain]=1;
	}
    }
}
$finishTime = microtime(true);
DisplayExecutionTime($queryTime, $finishTime, "Iterating Port table");

// No additional processing necessary if there are no HA pairs
if( $nodeBD ){

    $removeStartTime = microtime(true);
    // remove every broadcast domain the HA pair has in common 
    foreach ($nodeBD as $node=>$ipspaces){
	$partnerNode = $partner[$node];
	foreach ($ipspaces as $ipspace=>$bDomains) {
	    foreach ($bDomains as $broadcastDomain=>$value) {
		if ($nodeBD[$partnerNode][$ipspace][$broadcastDomain]) {
		    unset( $nodeBD[$partnerNode][$ipspace][$broadcastDomain] );
		    unset( $nodeBD[$node][$ipspace][$broadcastDomain] );

		    # if no more broadcast domains exist, remove the
		    # parent entry for both the node and it's partner
		    if( 0 == count( $nodeBD[$node][$ipspace] )){
			unset( $nodeBD[$node][$ipspace] );
			if( 0 == count( $nodeBD[$node] )){
			    unset( $nodeBD[$node] );
			}
		    }
		    if( 0 == count( $nodeBD[$partnerNode][$ipspace] )){
			unset( $nodeBD[$partnerNode][$ipspace] );
			if( 0 == count( $nodeBD[$partnerNode] )){
			    unset( $nodeBD[$partnerNode] );
			}
		    }
		}
	    }
	} 
    }
    DisplayExecutionTime( $removeStartTime, microtime(true), "clearing out matching BDs");
}
if( $nodeBD ){
    // at this point, nodeBD contains the broadcast domains that are only
    // on one node of each HA pair.  This isn't necessarily a problem if the 
    // only LIFS on that broadcast domain do not fail over.  So we need
    // to remove those broadcast domains from nodeBD
    unset ($errorMsg);
    reset ($nodeBD);
    
    $startLifCheckTime = microtime(true);

    // remove any broadcast domain that doesn't host lifs that fail over
    foreach ($nodeBD as $node=>$ipspaces){
	foreach ($ipspaces as $ipspace=>$bDomains) {
	    foreach ($bDomains as $bdId=>$value) {
		$mustExistOnPartner = False;

		// get the ports in the broadcast domain local to the node
		$portQuery= "SELECT port_id_pkey from broadcast_domain_port_akey where node_id_col = '$node' and broadcast_domain_id_col=$bdId";
		$portResult = dotsql_query($portQuery);
		while($portRow = dotsql_fetch_object($portResult)){
		    $portId = $portRow->{'port_id_pkey'};

		    // check if there is a lif that can fail over on any port in that broadcast domain
		    // we only want data or cluster-mgmt, unfortunately, OR doesn't work for dotsql so
		    // we need to filter out what we don't want using AND.
		    // also, we only want to consider admin-up lifs on running vservers
		    // also disregard lifs with a failover policy of local-only 
		    $lifQuery = "SELECT vsrv_id_col from port_vif_akey WHERE current_port_fkey=$portId and vif_type_col<>node-mgmt and vif_type_col<>cluster and vif_type_col<>intercluster and vif_status_col=up and failover_policy_col<>local-only and failover_policy_col<>disabled";
		    $lifResult = dotsql_query($lifQuery);
		    while($lifRow = dotsql_fetch_object($lifResult)){
			$vserverId = $lifRow->{'vsrv_id_col'};

			// check if the vserver is running
			// get the vserver name for the id and check if it is running
			$vserverQuery = "SELECT name FROM sequential_identifier WHERE namespace='virtual.servers' AND id=$vserverId";
			$vserverResult = dotsql_query($vserverQuery);
			if( $vserverRow = dotsql_fetch_object($vserverResult) ){
			    $vserverName = $vserverRow->{'name'};
			    list($vserverState,$vserverType) = getVserverInfo($vserverName);
			    if( $vserverState == "running") {
				$mustExistOnPartner = True;
				break;
			    }
			    else{
			    }
			}
			else{
			    // this is the system vserver of the default ipspace which
			    // is always running
			}
		    }
		}

		// remove the broadcast domain if no lifs that can fail over use it
		if ($mustExistOnPartner == False) {
		    unset( $nodeBD[$node][$ipspace][$bdId] );

		    // if no more broadcast domains exist, remove the
		    // parent entry for both the node and it's partner
		    if( 0 == count( $nodeBD[$node][$ipspace] )){
			unset( $nodeBD[$node][$ipspace] );
			if( 0 == count( $nodeBD[$node] )){
			    unset( $nodeBD[$node] );
			}
		    }
		}
	    }
	}
    }
    DisplayExecutionTime($startLifCheckTime, microtime(true), "Lif check");
}

if($nodeBD) {
    unset ($errorMsg);
    reset ($nodeBD);

    $mismatchNodeCount = 0;
    $totalMismatchBDomains = 0;

    $startFindMismatchTime = microtime(true);

    // we want to report the first MAX_BDOMAINS_TO_REPORT
    // broadcast domains, the remaining number of nodes
    // that has missing broadcast domains without any
    // mention of the specific broadcast domain missing to
    // conserve space in the output string

    // any broadcast domains remaining are not present on its partner
    // if there are no mismatched broadcast domains, the $nodeBD
    // will be empty
    foreach ($nodeBD as $node=>$ipspaces){

	if( $totalMismatchBDomains <= $MAX_BDOMAINS_TO_REPORT){

	    unset ($missingBDomains);
	    unset ($missingBDomainCount);
	    $firstMissingBD = 1;

	    foreach ($ipspaces as $ipspace=>$bDomains){

		foreach ($bDomains as $bdId=>$value) {

		    // convert bdId to broadcast domain name
		    $bdSeqIdQuery = "SELECT name FROM vifmgr_seqids WHERE id={$bdId} AND namespace = 'bcst_domains.table'";
		    $bdSeqIdResult = dotsql_query($bdSeqIdQuery);
		    $bdSeqIdRow = dotsql_fetch_object($bdSeqIdResult);
		    $ipspaceBDName = $bdSeqIdRow->{'name'};

		    if( $ipspaceBDName ){
			// parse the ipspace and bcast domain name out format is "IPspace4294967294_Broadcast_DomainCluster";
			if( 1 != preg_match('/IPspace(\d+)_Broadcast_Domain(.+)$/', $ipspaceBDName,$matches)) {
			    // could not parse the ipspace/broadcast domain name
			    // ignore and go to next missing bd domain
			    continue;
			}
			else {
			    $ipspaceId = $matches[1];
			    $broadcastDomain = $matches[2];

			    // get ipspace name from ipspace id
			    $bdIpspaceQuery = "SELECT ipspace FROM ipspaces_table WHERE id = {$ipspaceId}";
			    $bdIpspaceNameResult = dotsql_query($bdIpspaceQuery);
			    $bdIpspaceNameRow = dotsql_fetch_object($bdIpspaceNameResult);
			    $ipspaceName = $bdIpspaceNameRow->{'ipspace'};
			    if( $ipspaceName ) {

				$totalMismatchBDomains++;

				if( $totalMismatchBDomains <= $MAX_BDOMAINS_TO_REPORT){
				    if( $nodeBD[$node][$ipspace][$bdId] ) {

					if( $firstMissingBD == 1 ){
					    $firstMissingBD = 0;
					}
					else {
					    $missingBDomains .= ","; 
					}

					$missingBDomains .= "\"{$broadcastDomain}\" in IPspace \"{$ipspaceName}\"";

					$missingBDomainCount++;
				    }
				}
			    }
			}
		    }
		}
	    }

	    if( $missingBDomains ){
		$msg = "Node \"$partner[$node]\" needs at least one port configured in the broadcast domain%s %s. ";
		if ( $missingBDomainCount == 1 ) {
		    $multi="";
		}
		else{
		    $multi="s";
		}
		$errorMsg .= sprintf( $msg, $multi, $missingBDomains );
	    }
	}
	else{
	    $mismatchNodeCount++;
	}

    }

    DisplayExecutionTime($startFindMismatchTime, microtime(true), "find mismatch");

    if( $mismatchNodeCount > 0){
	// this is where there are more nodes that have mismatched bdomains
	$errorMsg .= " There are $mismatchNodeCount additional nodes that have ports in broadcast domains not present on their partner.";
    }

    if( $errorMsg ){
	echo "<Alert>$errorMsg</Alert>";
	echo "<Action>To find the HA Pairs, use the \"storage failover show -fields partner-name\" command. To display ";
	echo "each port's broadcast-domain, use the \"network port show -node <node-name> -fields broadcast-domain\" command. ";
	echo "Broadcast-domain commands are found in the \"network port broadcast-domain\" command set.</Action>";
	echo "<Advice>Modify the broadcast domains to match the ports on HA pairs</Advice>";
	echo "<RC>warning</RC>";
    }
}

exit;
?>
