<?php
//
//  Copyright (c) 2019 NetApp, Inc.
//  All rights reserved.
//
// Check for metrocluster un-mirrored aggregates. This file checks:
// 1. You must mirror the root aggregates to ensure data protection.
//    Check aggr-status -v for mirrored keyword in raidstatus field.
// 2. MetroCluster Storage Virtual Machine's (SVM) root volume must be on a mirrored aggregate:
//	  Verify that the root aggregates where the SVMs are hosted are mirrored.
// 3. MetroCluster MetaDataVolume must be hosted on a mirrored aggregate: Verify no "MDV_CRS_*" is on an unmirrored aggregate
//


//Check if root aggregates are mirrored or not
$queryaggr = "SELECT aggregate, raidstatus FROM aggr WHERE root = true AND raidstatus LIKE '%mirror%degraded%' LIMIT 1";
$resultaggr = dotsql_query($queryaggr);
$is_warning = false;

if (dotsql_num_rows($resultaggr) > 0) {
		$alert = "Root aggregates are not mirrored in MetroCluster configuration.";
		$action = "To check aggregate status, run the \"storage aggregate show -root true\" command.";
		$advice = "Root aggregates should be mirrored in a MetroCluster configuration.";
		$is_warning = true;
}

//Check if the root aggr of vservers are mirrored or not
$aggregates = array();
$queryvserver = "SELECT aggregate,rootvolume FROM vserver_ui";
$resultvserver = dotsql_query($queryvserver);
while ($rowvserver = dotsql_fetch_object($resultvserver)) {
	 $root_aggr = $rowvserver->{'aggregate'};
	 if ($rowvserver->{'rootvolume'} && !in_array($root_aggr,$aggregates)) {
		array_push($aggregates, $root_aggr);
		$queryaggr = "SELECT raidstatus FROM aggr WHERE aggregate = $root_aggr AND raidstatus LIKE '%mirror%degraded%'";
	        $resultaggr = dotsql_query($queryaggr);

		if (dotsql_num_rows($resultaggr) > 0) {
			$alert .= " Vserver root aggregates are not mirrored in MetroCluster configuration.";
			$action .= " To check Vserver root volume, run the \"vserver show\" command. To check the aggregate, the root volume belongs to use the \"volume show\" command. Then check the aggregate status using the \"storage aggregate show\" command.";
			$advice .= " Vserver root aggregates should be mirrored in MetroCluster configuration.";
			$is_warning = true;
			break;
		}
	 } else {
		continue;
	}
}

//Check whether MCC metadata volumes(MDV_CRS_* vols) lie in mirrored aggregates or not.
$queryvol = "SELECT aggregate FROM volume WHERE volume LIKE MDV_CRS_%";
$resultvol = dotsql_query($queryvol);

while ($rowaggr = dotsql_fetch_object($resultvol)) {
	$aggr = $rowaggr->{'aggregate'};
	$queryaggr = "SELECT raidstatus FROM aggr WHERE aggregate = '".$aggr."' AND raidstatus LIKE '%mirror%degraded%' LIMIT 1";
	$resultaggr = dotsql_query($queryaggr);

	if (dotsql_num_rows($resultaggr) > 0) {
				$alert .= " MetroCluster metadata volumes are not on mirrored aggregates.";
				$action .= " To check the aggregate, the MetroCluster metadata volume belongs to use the \"volume show\" command. Then check the aggregate status using the \"storage aggregate show\" command.";
				$advice .= " Metadata volumes should be on aggregates which are mirrored in MetroCluster configuration.";
				$is_warning = true;
				break;
	}
}


if ($is_warning) {
	echo "<Alert>".$alert."</Alert>";
	echo "<Action>".$action."</Action>";
	echo "<Advice>".$advice."</Advice>";
	echo "<RC>error</RC>";
}


exit;
?>

