<?php

// Copyright (c) 2016 NetApp, Inc.
// All rights reserved.

/*
 * Upgrade/Revert Guide contents:
 *
 * If your cluster serves NFS clients, you must use hard mounts.
 * ------------------------------------------------------------
 * You should not use soft mounts when there is a possibility of frequent NFS
 * timeouts, which can lead to disruptions during the upgrade process and
 * possible data corruption.
 *
 * Considerations for stateless protocols
 * --------------------------------------
 * Configurations that include client connections using stateless NAS and SAN
 * protocols generally do not experience adverse effects during upgrades if the
 * clients are configured according to recommended guidelines.
 *
 * If you are using stateless protocols, consider the following:
 * - NFS hard mounts
 *   No adverse behavior is experienced on the clients during upgrade. Clients
 *   might receive some messages similar to the following until node reboots:
 *       NFS server not responding, retrying
 *   In general, read/write directories should be hard-mounted. Hard mounts are
 *   the default type of mount.
 * - NFS soft mounts
 *   You should not use soft mounts when there is a possibility of frequent NFS
 *   timeouts. Race conditions can occur as a result of these timeouts, which
 *   can lead to data corruption. Furthermore, some applications cannot properly
 *   handle errors that occur when an NFS operation reaches a timeout using soft
 *   mounts.
 *       Situations that can cause frequent timeouts include nondisruptive
 *   upgrades or any takeover or giveback event in an HA configuration.
 *       In general, soft mounts should be used only when reading solely from a
 *   disk; even then, understand that any soft mount is unreliable.
 */

/*
 * DOTSQL query output:
 * --------------------
 * dotsql> SELECT vserver, access, v3, v4.0, v4.1 FROM nfs_servers_byname
 * vserver access v3       v4.0     v4.1
 * ------- ------ -------- -------- --------
 * vs0     true   enabled  disabled disabled
 * vs1     true   disabled disabled disabled
 * vs2     false  enabled  disabled disabled
 *
 * Ideally we should be checking for both access field and different NFS
 * protocol version fields v3, v4.0 and v4.1 to see if they are true
 * and enabled.
 *   However, in setups with a large number of vservers, this can cause
 * this PHP script to timeout and leading to validation failure.
 *   So a minimalistic approach is considered wherein if the access field
 * of any vserver is true we consider that NFS is enabled in the cluster.
 */

// Messages displayed during validation.
$error_msg  = "This cluster is serving NFS clients. If NFS soft mounts are used, "
            . "there is a possibility of frequent NFS timeouts and race conditions "
            . "that can lead to data corruption during the upgrade.";

$action_msg = "Use NFS hard mounts, if possible. To list Vservers running NFS, "
            . "run the following command: vserver nfs show";

$advice_msg = "Use NFS hard mounts, if possbile.";

$query = "SELECT vserver FROM nfs_servers_byname where access = true LIMIT 1";
$query_result = dotsql_query($query);

if (dotsql_num_rows($query_result)) {
    echo "<Alert>$error_msg</Alert>";
    echo "<Action>$action_msg</Action>";
    echo "<Advice>$advice_msg</Advice>";
    echo "<RC>warning</RC>";
}

?>
