<?php
//  Copyright (c) 2017 NetApp, Inc.
//  All rights reserved.

// NDU Compatibility for Openssh 7.2 Upgrade
// VERSION="1.0"
// This script checks whether weaker mac algorithms(hmac-ripemd160 and hmac-ripemd160-etm)
// are configured for the vserver/cserver and warns the user that the support for these
// mac algorithms will be  removed post upgrade to FT. These mac algorithms needs to be
// explicitly removed from the vserver using security ssh remove command.

include 'mcc_util.php';

// All the validations only apply for major upgrades.
// Exit the validation in minor upgrades.
$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];
if($majorupgrade == "false") {
    exit;
}

// Check whether HMAC algorithms are configured for the vserver/cserver.
$query = "SELECT vserver FROM ssh WHERE mac-algorithms=hmac-ripemd160* LIMIT 1";
$result = dotsql_query($query);

// If the vserver found with weaker HMAC algorithms configured,
// then warn the user.
if(dotsql_num_rows($result) > 0) {
    $log_message = "Vservers having weaker HMAC algorithms configured found.";
    warnUser($vserver_having_buggy_mac_algo_array);
} else {
    $log_message = "Vservers having weaker HMAC algorithms configured not found. ";
    logDebugMessage($log_message);
}

//--------------------------------FUNCTION DEFINATION---------------------------
function logDebugMessage($log_message)
{
    $dbgquery = "CALL add_scripting_log_message('validation_rules_openssh_7_2: $log_message','false')";
    $result = dotsql_query($dbgquery);
}

function warnUser($vserver_having_buggy_mac_algo_array)
{
    logDebugMessage($log_message);
    echo "<Alert>";
    echo "\"hmac-ripemd160\" and \"hmac-ripemd160-etm\" are ";
    echo "considered weak keyed-hash message authentication ";
    echo "code (HMAC) algorithms and support for the same ";
    echo "will be removed after upgrading to Data ONTAP 9.3. ";
    echo "</Alert><Action>";
    echo "Before retrying the upgrade, remove the ";
    echo "above weak algorithms using \"security ssh remove\" command. ";
    echo "To list all Vservers configured with one or both the ";
    echo "above HMAC algorithms, run \"security ssh show -mac-algorithms ";
    echo "hmac-ripemd160* -vserver * -fields vserver\" . ";
    echo "</Action><Advice>";
    echo "Ensure the Vservers do not have these HMAC algorithms configured ";
    echo "before attempting to upgrade.";
    echo "</Advice>";
    echo "<RC>warning</RC>";
}

//---------------------------------------------------------------------------------
exit;
?>
