<?php
//  Copyright (c) 2017 NetApp, Inc.
//  All rights reserved.

// NDU Compatibility for OpenSSH MAC Algorithms.
// VERSION="1.0"
// This script checks whether SHA-1 or MD5 MAC Algorithms are enabled,
// and issues warning if they are. NDU will not be allowed if no secure
// MAC Algorithms are configured as unsecure MAC Algorithms will be
// removed from SSH configuration during upgrade.
//
include 'mcc_util.php';

// All the validations only apply for major upgrades.
// Exit the validation in minor upgrades.
$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];
if($majorupgrade == "false") {
    exit;
}

$ecvQuery = "SELECT generation,major FROM effective_cluster_version";
$result = dotsql_query($ecvQuery);
$row = dotsql_fetch_object($result);
if( $row ){
    $gen = $row->{'generation'};
    $major = $row->{'major'};
    if( !( $gen > 9 || ( $gen >= 9 && $major >= 11 ) ) ){
	exit;
    }
}

$vserver_name = " ";
$unsecure_mac_present = false;
$unsecure_mac_only = false;
$vserver_query = "SELECT id,vserver FROM vserver_by_name_data_admin";
$vserver_result = dotsql_query($vserver_query);
while( $vserver_row = dotsql_fetch_object($vserver_result)){
    $vserver = $vserver_row->{'id'};
    $ssh_query = "SELECT mac-algorithms FROM ssh WHERE vserver=$vserver";
    $ssh_result = dotsql_query($ssh_query);
    $ssh_row = dotsql_fetch_object($ssh_result);
    $mac_algorithms = $ssh_row->{'mac-algorithms'};
    $mac_count = substr_count($mac_algorithms, ',');
    $mac_count = $mac_count + 1;
    $sha1_mac_count = substr_count($mac_algorithms, 'sha1');
    $md5_mac_count = substr_count($mac_algorithms, 'md5');
    $unsecure_mac_count = $sha1_mac_count + $md5_mac_count;
    if($unsecure_mac_count > 0){
	$unsecure_mac_present = true;
	if($mac_count == $unsecure_mac_count) {
		$unsecure_mac_only = true;
		$vserver_name = $vserver_row->{'vserver'};
	    break;
	}
    }
}

$fips_enabled = false;
$fips_query = "SELECT is-fips-enabled FROM security_config_ui";
$fips_result = dotsql_query($fips_query);
$fips_row = dotsql_fetch_object($fips_result);
if($fips_row->{'is-fips-enabled'} == "true") {
    $fips_enabled = true;
}

if($unsecure_mac_present || $unsecure_mac_only) {
    echo "<Alert>";
    echo "Vserver found with SHA-1/MD5 MAC Algorithm. ";
    echo "SHA-1/MD5 MAC Algorithms are considered weak, ";
    echo "and will be removed upon upgrade.";
    echo "</Alert>";
    echo "<Action>";
    if($unsecure_mac_only) {
	echo "Add atleast one of the below secure MAC Algorithms ";
	echo "to the Vserver \"$vserver_name\" before retyring the upgrade: ";
	echo "hmac-sha2-512, hmac-sha2-512-etm, hmac-sha2-256,";
	echo "hmac-sha2-256-etm";
	if(!($fips_enabled)) {
	    echo ", umac-64, umac-64-etm, umac-128, umac-128-etm";
	}
    }
    echo "</Action>";
    echo "<Advice>";
    echo "Ensure that all the Vservers are configured with ";
    echo "secure MAC Algorithms.";
    echo "</Advice>";
    echo "<RC>";
    if($unsecure_mac_only){
	echo "error";
    } else {
	echo "warning";
    }
    echo "</RC>";
}

exit;
?>
