<?php
//
//  Copyright (c) 2019 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

//
// Check if all fields are present in smf_schema_replicated_view
//

function query_field_in_table($table, $field, $process) {

    // Construct query and see if field has values
    $dslink = dotsql_connect("", "", 0);
    $ok = true;
    $query = "SELECT ".$field." FROM ".$table;
    $result = dotsql_query($query, $dslink);
    if ($result) {
        while ($row = dotsql_fetch_object($result)) {
            // Must be all ""'s
            $fieldv = $row->{$field};
            if ($fieldv != "") {
                dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK ERROR: schema for table '".$table
                           ."' in process '".$process."' has missing attribute(s) for field '"
                           .$field."' where sql-type-map-version is 1 and field is populated.\n");
                $ok = false;
                break;
            }
        }

        $err = dotsql_error($dslink);
        if ($err != "") {
            dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK ERROR: cannot query through table '".$table
                       ."' in process '".$process."' for missing attribute(s) "
                       ."where sql-type-map-version is 1. ERR: $err\n");
            $ok = false;
        }
    } else {
        $err = dotsql_error($dslink);
        if ($err != "") {
            $errno = dotsql_errno($dslink);
            if ($errno == 28) {
                // Will fall into here if field has been removed from table, but exists in cluster schema
                dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK NOTICE: cannot query table '".$table
                           ."' in process '".$process."'. Table does not have field '".$field."'. Continuing.\n");
            } else {

                dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK ERROR: cannot query table '".$table
                           ."' in process '".$process."' for missing attribute(s) "
                           ."where sql-type-map-version is 1. ERR: $err\n");
                $ok = false;
            }
        }
    }

    dotsql_close($dslink);
    return $ok;
}

function check_smf_schema_consistency() {

    $ok = true;

    $link = dotsql_connect("", "", 0);
    $query = "SELECT process, table-name, field-pid, sql-type, sql-type-map-version, smf-type, rdb-type, field-name, is-list, persistence-type FROM smf_schema_replicated_view";
    $result = dotsql_query($query, $link);
    while ($row = dotsql_fetch_object($result)) {
        if ($row->{'field-pid'} != "0") {
            $typemapver = intval($row->{'sql-type-map-version'});
            if ($typemapver >= 2) {
                if ($row->{'sql-type'} == "" ||
                    $row->{'smf-type'} == "" ||
                    $row->{'rdb-type'} == "" ||
                    $row->{'field-name'} == "" ||
                    $row->{'is-list'} == "" ||
                    $row->{'persistence-type'} == "") {
                    // All information must be present for sql-type-map-version 2 or greater
                    dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK ERROR: schema for table '".$row->{'table-name'}
                               ."' in process '".$row->{'process'}."' has missing attribute(s) for field '"
                               .$row->{'field-name'}."' where sql-type-map-version is 2 or greater.\n");
                    $ok = false;
                    break;
                }
            } else if ($typemapver == 1) {
                // Check for any missing attributes
                if ($row->{'sql-type'} == "" ||
                    $row->{'smf-type'} == "" ||
                    $row->{'rdb-type'} == "" ||
                    $row->{'field-name'} == "" ||
                    $row->{'is-list'} == "" ||
                    $row->{'persistence-type'} == "") {

                    $ok = query_field_in_table($row->{'table-name'}, $row->{'field-name'}, $row->{'process'});
                    if ($ok === false) {
                        break;
                    }
                }
            } else {
                // Unknown value
                $ok = false;
                break;
            }
        }
    }

    $err = dotsql_error($link);
    if ($err != "" && $ok == true) {
        dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK ERROR: cannot query through SMF schema. ERR: $err\n");
        $ok = false;
    }

    if ($ok === false) {
        echo "<Alert>Cannot upgrade due to replicated database inconsistency</Alert> <Action>Contact Support.</Action> ";
        echo "<Advice>Contact Support.</Advice>";
        echo "<RC>error</RC>";
        exit;
    }
    dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK NOTICE: Successfully completed.\n");
}

function check_script_running_in_95orlater() {
    $query = "SELECT generation, major FROM effective_cluster_version";
    $result = dotsql_query($query);
    $row = dotsql_fetch_object($result);

    if (!$row || ($row && ($row->{'generation'} == '' || $row->{'major'} == ''))) {
        echo "<Alert>Cannot upgrade due to failure in acquiring cluster version</Alert> <Action>Contact Support.</Action> ";
        echo "<Advice>Contact Support.</Advice>";
        echo "<RC>error</RC>";
        exit;
    }

    $gen = intval($row->{'generation'});
    $maj = intval($row->{'major'});

    if (($gen == 9 && $maj >= 5) || $gen > 9) {
        dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK NOTICE: Executing check as cluster version is 9.5 or later.\n");
        return true;
    } else {
        dotlog_msg(DOTLOG_ERR, "messages", "RDB ANDU PRE-CHECK NOTICE: Not executing check as cluster version is less than 9.5.\n");
        return false;
    }
}

if (check_script_running_in_95orlater()) {
    check_smf_schema_consistency();
}
?>
