<?php
//
//  Copyright (c) 2018 - 2022 NetApp, Inc.
//  All rights reserved.
//

// VERSION="1.0"

//
// iSCSI tpgroups are not going to be supported in 9.13.0+
//
// We'll clean up any remaining tpgroups and replace them with the underlying
// LIFs in any configurd portsets  during the upgrade, but it forces
// a complete purge and reload of the Vserver, which could be disruptive
// to all blocks protocols in the Vserver depending on timing. If the user
// unwinds them manually, it will only disrupt a single iSCSI path at a time,
// but can be annoyingly difficult to unwind if there's a tangle of tpgroups
// and portsets.
//
// OTOH, we've had this warning here for 4 years, so there really shouldn't
// be anyone left in the field running these. Somewhat debatable if we should
// leave this as a warning, or force it to an error and only do our cleanup
// if they manage to bypass this completely.

// Get all user-defined target portal groups
$query = "SELECT vserver,tpgroup FROM tpgroup WHERE is-user-defined=true";
$result = dotsql_query($query);
$msg = "";
while ($row = dotsql_fetch_object($result)) {
    if ($msg) {
        $msg .= ", ";
    }
    $msg .= $row->vserver . ":" . $row->tpgroup;
}

if ($msg) {
    echo <<<MESSAGE
    <Alert>
        User-defined iSCSI target portal groups are no longer supported in
	ONTAP and will be removed as part of the upgrade process. It is
	recommended to manually remove all user-defined iSCSI target portal
	groups before proceeding (vserver:tpgroup): ${msg}
    </Alert>
    <Action>
        Contact support for help removing all user-defined iSCSI target
        portal groups.
    </Action>
    <Advice>
        Contact support for help removing all user-defined iSCSI target
        portal groups.
    </Advice>
    <RC>warning</RC>
MESSAGE;
}
exit;
?>
