<?php

// Copyright (c) 2016-2022 NetApp, Inc.
// All rights reserved.

// Warn the user to review cluster/management switches before upgrade/downgrade.

/*
 * Upgrade/Revert Guide contents:
 *
 * Section: Reviewing pre-upgrade resources
 * Confirm that your cluster and management switches are supported in the target
 * release.
 * -----------------------------------------------------------------------------
 * You must verify that the cluster network switches, storage network switches,
 * IOS (management network switches), and reference configuration file (RCF)
 * software versions are compatible with the version of Data ONTAP to which you
 * are upgrading. If your switches do not have the minimum software versions
 * for the target Data ONTAP release, then you must upgrade the switch software
 * before you can upgrade Data ONTAP
 *
 * Section: Reviewing pre-downgrade resources
 * Confirm that your cluster and management switches are supported in the target
 * release.
 * -----------------------------------------------------------------------------
 * You must verify that the cluster network switches, storage network switches,
 * IOS (management network switches), and reference configuration file (RCF)
 * software versions are compatible with the version of Data ONTAP to which
 * you are downgrading.
 *
 * URL: http://mysupport.netapp.com/NOW/download/software/cm_switches/
 *
 * NOTE: Since ANDU doesn't support revert, pre-reversion section information
 * for cluster and management switches is not considered.
 */

include 'mcc_util.php';

function logDebugMessage($message) {
    $dbgquery = "CALL add_scripting_log_message('validation_rules_switch_support: $message', 'false')";
    $result = dotsql_query($dbgquery);
}

$options = getoptreq('', array("major:", "downgrade::"));
$majorupgrade = $options["major"];

if ($majorupgrade == "false") {
    logDebugMessage("Minor upgrade. Skipping the check.");
    exit;
}

// Messages displayed during validation.
$error_msg  = "Ensure that the cluster network switches, storage network "
            . "switches, IOS (management network switches) software and "
            . "reference configuration file (RCF) versions are compatible "
            . "with the target Data ONTAP release.";

$action_msg = "Refer to https://mysupport.netapp.com/site/downloads "
            . "and the NetApp Hardware Universe for more details.";

$advice_msg = "Refer to https://mysupport.netapp.com/site/downloads "
            . "and the NetApp Hardware Universe for more details.";

$action_msg_lenovo = "Refer to https://datacentersupport.lenovo.com/ for more details. ";

$advice_msg_lenovo = "Refer to https://datacentersupport.lenovo.com/ for more details.";

$vsim_family = 'NetApp VSim';
$vsa_family = 'Virtual Machine';

$query= "SELECT board-type,vendor-id FROM system_node_info";
$result = dotsql_query($query);
$row = dotsql_fetch_object($result);
$board_type = $row->{'board-type'};
$vendor_id = $row->{'vendor-id'};
logDebugMessage("board type is $board_type");
logDebugMessage("vendor is $vendor_id");

// Display message only for real hardware models i.e., FAS or AFF systems
if ((strcasecmp($board_type, $vsim_family) != 0) &&
       (strcasecmp($board_type, $vsa_family) != 0))
{
   if($vendor_id == "Lenovo")
   {
	logDebugMessage("Vendor id is Lenovo");
	echo "<Alert>$error_msg</Alert>";
	echo "<Action>$action_msg_lenovo</Action>";
	echo "<Advice>$advice_msg_lenovo</Advice>";
	echo "<RC>warning</RC>";
   }
   else
   {
	logDebugMessage("Vendor id is Netapp");
	echo "<Alert>$error_msg</Alert>";
	echo "<Action>$action_msg</Action>";
	echo "<Advice>$advice_msg</Advice>";
	echo "<RC>warning</RC>";

   }
}
?>
