<?php
// Copyright (c) 2022 NetApp, Inc.
// All rights reserved.

// OpenSSL 3.0.0 NDU Compatibility

// This script checks if TLSv1.1 is enabled alongside FIPS mode and warns the
// user that TLSv1.1 is no longer supported in FIPS mode and will be disabled
// post-upgrade

// VERSION="1.0"

$query = "SELECT interface FROM security_config WHERE is-fips-enabled = true AND supported-protocols LIKE '%TLSv1.1%'";
$result = dotsql_query($query);


if(dotsql_num_rows($result) > 0) {
    echo "<Alert>";
    echo "TLSv1.1 is no longer supported in FIPS mode and will be disabled ";
    echo "after upgrading to ONTAP 9.11.1.";
    echo "</Alert><Action>";
    echo "Before retrying the upgrade, disable TLSv1.1 or FIPS mode using the ";
    echo "\"security config modify\" command.";
    echo "</Action><Advice>";
    echo "Ensure TLSv1.1 and FIPS mode are not both enabled before attempting ";
    echo "to upgrade.";
    echo "</Advice>";
    echo "<RC>warning</RC>";
}

exit;
?>
